@@ -472,7 +472,7 @@ impl ChannelBuilder {
     }
 }
 
-#[cfg(feature = "secure")]
+#[cfg(feature = "_secure")]
 mod secure_channel {
     use std::borrow::Cow;
     use std::ffi::CString;
@@ -570,6 +570,7 @@ pub struct Channel {
     cq: CompletionQueue,
 }
 
+#[allow(clippy::non_send_fields_in_send_ty)]
 unsafe impl Send for Channel {}
 unsafe impl Sync for Channel {}
 
